"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ForgetPasswordController = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
const core_2 = require("@sourceloop/core");
const jwt = tslib_1.__importStar(require("jsonwebtoken"));
const loopback4_authentication_1 = require("loopback4-authentication");
const loopback4_authorization_1 = require("loopback4-authorization");
const models_1 = require("../models");
const repositories_1 = require("../repositories");
let ForgetPasswordController = /** @class */ (() => {
    let ForgetPasswordController = class ForgetPasswordController {
        constructor(userRepo, revokedTokensRepo) {
            this.userRepo = userRepo;
            this.revokedTokensRepo = revokedTokensRepo;
        }
        async forgetPassword(req, client) {
            var _a;
            const user = await this.userRepo.findOne({
                where: {
                    username: req.username,
                },
            });
            if (!user || !user.id) {
                throw new rest_1.HttpErrors.NotFound('User not found !');
            }
            if (!user.email) {
                throw new rest_1.HttpErrors.UnprocessableEntity('No email exists for the user !');
            }
            try {
                const codePayload = {
                    clientId: client.clientId,
                    userId: parseInt(user.id),
                    user: new models_1.User({
                        id: user.id,
                        email: user.email,
                        username: user.username,
                    }),
                };
                // Default expiry is 30 minutes
                const expiryDuration = parseInt((_a = process.env.FORGOT_PASSWORD_LINK_EXPIRY) !== null && _a !== void 0 ? _a : '1800');
                const token = jwt.sign(codePayload, process.env.JWT_SECRET, {
                    expiresIn: expiryDuration,
                    audience: req.client_id,
                    subject: user.username.toLowerCase(),
                    issuer: process.env.JWT_ISSUER,
                });
                return new models_1.ForgetPasswordResponseDto({
                    code: token,
                    expiry: expiryDuration,
                    email: user.email,
                    user,
                });
            }
            catch (error) {
                throw new rest_1.HttpErrors.UnprocessableEntity("Client Verification Failed" /* ClientVerificationFailed */);
            }
        }
        async verifyResetPasswordLink(token) {
            let payload;
            const isRevoked = await this.revokedTokensRepo.get(token);
            if (isRevoked === null || isRevoked === void 0 ? void 0 : isRevoked.token) {
                throw new rest_1.HttpErrors.Unauthorized("TokenRevoked" /* TokenRevoked */);
            }
            try {
                payload = jwt.verify(token, process.env.JWT_SECRET, {
                    issuer: process.env.JWT_ISSUER,
                });
            }
            catch (error) {
                throw new rest_1.HttpErrors.Unauthorized("Token Expired" /* TokenExpired */);
            }
            if (!payload.clientId || !payload.user) {
                throw new rest_1.HttpErrors.Unauthorized("Token Invalid" /* TokenInvalid */);
            }
            return new core_2.SuccessResponse({
                success: true,
            });
        }
        async resetPassword(req, client) {
            if (!req.token) {
                throw new rest_1.HttpErrors.UnprocessableEntity("TokenMissing" /* TokenMissing */);
            }
            if (req.password && req.password.length <= 0) {
                throw new rest_1.HttpErrors.BadRequest("PasswordInvalid" /* PasswordInvalid */);
            }
            let payload;
            const isRevoked = await this.revokedTokensRepo.get(req.token);
            if (isRevoked === null || isRevoked === void 0 ? void 0 : isRevoked.token) {
                throw new rest_1.HttpErrors.Unauthorized("TokenRevoked" /* TokenRevoked */);
            }
            try {
                payload = jwt.verify(req.token, process.env.JWT_SECRET, {
                    issuer: process.env.JWT_ISSUER,
                });
            }
            catch (error) {
                throw new rest_1.HttpErrors.Unauthorized("Token Expired" /* TokenExpired */);
            }
            if (!payload.clientId || !payload.user) {
                throw new rest_1.HttpErrors.Unauthorized("Token Invalid" /* TokenInvalid */);
            }
            await this.userRepo.changePassword(payload.user.username, req.password);
            await this.revokedTokensRepo.set(req.token, {
                token: req.token,
            });
        }
    };
    tslib_1.__decorate([
        loopback4_authentication_1.authenticateClient("client-password" /* CLIENT_PASSWORD */),
        loopback4_authorization_1.authorize(['*']),
        rest_1.post(`auth/forget-password`, {
            responses: {
                [200 /* OK */]: {
                    description: 'Success Response.',
                    content: {
                        [core_2.CONTENT_TYPE.JSON]: {
                            schema: rest_1.getModelSchemaRef(models_1.ForgetPasswordResponseDto),
                        },
                    },
                },
                ...core_2.ErrorCodes,
            },
        }),
        tslib_1.__param(0, rest_1.requestBody()),
        tslib_1.__param(1, core_1.inject(loopback4_authentication_1.AuthenticationBindings.CURRENT_CLIENT)),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [models_1.ForgetPasswordDto,
            models_1.AuthClient]),
        tslib_1.__metadata("design:returntype", Promise)
    ], ForgetPasswordController.prototype, "forgetPassword", null);
    tslib_1.__decorate([
        loopback4_authorization_1.authorize(['*']),
        rest_1.get(`auth/verify-reset-password-link`, {
            responses: {
                [200 /* OK */]: {
                    description: 'Check if Token Is Valid and not Expired.',
                },
            },
        }),
        tslib_1.__param(0, rest_1.param.query.string('token', { required: true })),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [String]),
        tslib_1.__metadata("design:returntype", Promise)
    ], ForgetPasswordController.prototype, "verifyResetPasswordLink", null);
    tslib_1.__decorate([
        loopback4_authentication_1.authenticateClient("client-password" /* CLIENT_PASSWORD */),
        loopback4_authorization_1.authorize(['*']),
        rest_1.patch(`auth/reset-password`, {
            responses: {
                [204 /* NO_CONTENT */]: {
                    description: 'If User password successfully changed.',
                },
            },
        }),
        tslib_1.__param(0, rest_1.requestBody()),
        tslib_1.__param(1, core_1.inject(loopback4_authentication_1.AuthenticationBindings.CURRENT_CLIENT)),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [models_1.ResetPasswordWithClient,
            models_1.AuthClient]),
        tslib_1.__metadata("design:returntype", Promise)
    ], ForgetPasswordController.prototype, "resetPassword", null);
    ForgetPasswordController = tslib_1.__decorate([
        tslib_1.__param(0, repository_1.repository(repositories_1.UserRepository)),
        tslib_1.__param(1, repository_1.repository(repositories_1.RevokedTokenRepository)),
        tslib_1.__metadata("design:paramtypes", [repositories_1.UserRepository,
            repositories_1.RevokedTokenRepository])
    ], ForgetPasswordController);
    return ForgetPasswordController;
})();
exports.ForgetPasswordController = ForgetPasswordController;
//# sourceMappingURL=forget-password.controller.js.map